/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Benjamin Grauer
   co-programmer: ...
*/

//#define DEBUG_SPECIAL_MODULE DEBUG_MODULE_

#include "element_2d.h"
#include "render_2d.h"

#include "graphics_engine.h"
#include "p_node.h"

using namespace std;


/**
 * standard constructor
 * @todo this constructor is not jet implemented - do it
*/
Element2D::Element2D ()
{
  this->init();
}

/**
 * standard deconstructor
*/
Element2D::~Element2D ()
{
  // delete what has to be deleted here
  Render2D::getInstance()->unregisterElement2D(this);
}


void Element2D::init()
{
  this->setClassID(CL_ELEMENT_2D, "Element2D");

  this->setPosition2D(0,0);
  this->setAlignment(E2D_ALIGN_CENTER);

  Render2D::getInstance()->registerElement2D(this);
}

/**
 * this sets the position of the Element on the screen.
 * Use this in the your tick function, if you want the element to be automatically positioned.
 */
void Element2D::positioning()
{

  // setting the Position of this 2D-Element.
  if (this->alignment == E2D_ALIGN_SCREEN_CENTER)
  {
    absPos2D.x = GraphicsEngine::getInstance()->getResolutionX()/2 + this->relPos2D[0];
    absPos2D.y = GraphicsEngine::getInstance()->getResolutionY()/2 + this->relPos2D[1];
    absPos2D.depth = 0;
  }
  else if (this->bindNode)
  {
    GLdouble projectPos[3];
    gluProject(this->bindNode->getAbsCoor().x,
               this->bindNode->getAbsCoor().y,
               this->bindNode->getAbsCoor().z,
               GraphicsEngine::modMat,
               GraphicsEngine::projMat,
               GraphicsEngine::viewPort,
               projectPos,
               projectPos+1,
               projectPos+2);
    absPos2D.x = projectPos[0] + this->relPos2D[0];
    absPos2D.y = GraphicsEngine::getInstance()->getResolutionY() - projectPos[1] + this->relPos2D[1];
    absPos2D.depth = projectPos[2];
  }
  else
  {
    absPos2D.x = this->relPos2D[0];
    absPos2D.y = this->relPos2D[1];
    absPos2D.depth = 0;
  }
}

/**
 * ticks the 2d-Element
 * @param dt the time elapsed since the last tick
 */
void Element2D::tick(float dt)
{
  this->positioning();
}
